function [data, error] = MB_ReadData(addr, length)
%MODBUS ReadData(length)
%addr is the starting offset address
%length is number of bytes to read (must be modulo 2048)
%       max length is 65536
%   -1 returned for packet data if not mod 2048
%   -2 returned on error
%FC=4 : Read input Register
error = 0;
if (mod(length,2048) ~= 0)
    error = -1;
    data = 0;
    return
end

loops = length / 2048


t = tcpip('192.168.70.1',502);
set (t, 'InputBufferSize', 10000);
fopen(t);

data = [];

for x=0:loops-1
    %send modbus request
    header = [0 0 0 0];
    hdr_len = [0 6];
    targ_id = 1;
    func_code = 4;
    addr = [x*8 0];
    wordcntlo = 0;  %bitand(len,255)
    wordcnthi = 8;  %bitshift(bitand(len,65280),-8)
    word_cnt = [wordcnthi wordcntlo];
    command = [header hdr_len targ_id func_code addr word_cnt ]
    %header = [0 0  0 0  0 6  1 4 0 0 0 100]
    fwrite(t, command,'uint8');
    %wait for data from microblaze MODBUS server
    dlyloop = 0;
    while (((get(t, 'BytesAvailable')) == 0) && (dlyloop <= 1000))
        pause(.01);
        dlyloop = dlyloop + 1;
    end 
    disp(['waited : ', num2str(dlyloop)]);

    if (dlyloop < 1000) %didn't timeout
        bytes_rcvd = get(t, 'BytesAvailable')
        packet_data = uint8(fread(t,bytes_rcvd,'uint8'));
        numbytes = packet_data(9);
        if (bytes_rcvd == 4105)
            %strip off header
            rawdata = packet_data(10:bytes_rcvd);
            data16 = swapbytes(typecast(rawdata,'uint16'));
            %data_i = data16(1:4:length(data16));
            %data_q = data16(2:4:length(data16));
        else
            error = -2;
        end
    end
    data = [data;data16];
end
   
   
fclose(t);
disp('done')


